library mock_connection;

import 'dart:async';
import 'dart:collection';
import 'dart:typed_data';

import 'package:collection/wrappers.dart';
import 'package:fixnum/fixnum.dart';
import 'package:logging/logging.dart';

import '../src/connection.dart';
import '../src/schema_v1_pb2.dart';

class MockConnection implements DatastoreConnection {
  int _MAX_KEY_ID;

  List<int> allocatedIds = new List();


  @override
  Future<AllocateIdsResponse> allocateIds(AllocateIdsRequest request) {
    return new Future.sync(() {
      AllocateIdsResponse response = new AllocateIdsResponse();
      for (var key in request.key) {
        if (_isFullySpecifiedKey(key))
          throw 'Fully specified key: $key';
        var responseKey = new Key();
        for (var i=0;i<key.pathElement.length;i++) {
          var pe = key.pathElement[i];
          var peCopy = new Key_PathElement()
              ..kind = pe.kind;
          if (!pe.hasId() && !pe.hasName()) {
            var allocatedId = ++_MAX_KEY_ID;
            allocatedIds.add(allocatedId);
            peCopy.id = new Int64(allocatedId);
          } else {
            if (pe.hasId())
              peCopy.id = pe.id;
            if (pe.hasName())
              peCopy.name = pe.name;
          }
          responseKey.pathElement.add(peCopy);
        }
        response.key.add(responseKey);
      }
      return response;
    });
  }

  @override
  Future<BeginTransactionResponse> beginTransaction(BeginTransactionRequest request) {
    return _Transaction.beginTransaction().then((transaction) {
      BeginTransactionResponse response = new BeginTransactionResponse()
          ..transaction = transaction.id;
      return response;
    });
  }

  @override
  Future<CommitResponse> commit(CommitRequest request) {
    return _Transaction.commitTransaction(request.transaction, request.mutation)
        .then((_) {
    return new CommitResponse()
      ..mutationResult = (new MutationResult()..indexUpdates=0);
    });
  }



  @override
  Future<LookupResponse> lookup(LookupRequest request) {
    return new Future.sync(() {
      List<EntityResult> missing = [];
      List<EntityResult> results = [];
      for (var key in request.key) {
        var container = _lookupContainerByKey(key);
        if (container.isEmpty) {
          missing.add(new EntityResult()
              ..entity = (new Entity()..key = key));
        } else {

        }
      }
    });
  }

  @override
  Future<RollbackResponse> rollback(RollbackRequest request) {
    return _Transaction.rollbackTransaction(request.transaction)
        .then((_) => new RollbackResponse());
  }

  @override
  Future<RunQueryResponse> runQuery(RunQueryRequest request) {
    // TODO: implement runQuery
  }

  @override
  Future sendRemoteShutdown() {
    throw new UnimplementedError();
  }

  @override
  String get datasetId => throw new UnimplementedError();

  @override
  String get host => throw new UnimplementedError();

  @override
  Logger get logger => throw new UnimplementedError();

  @override
  Duration get timeoutDuration => throw new UnimplementedError();

  @override
  void set timeoutDuration(Duration _timeoutDuration) {
    throw new UnimplementedError();
  }
}