library entity_store_test;

import 'package:unittest/unittest.dart';
import 'package:fixnum/fixnum.dart';
import '../../lib/testing/src/entity_store.dart';
import '../../lib/src/schema_v1_pb2.dart';

void main() {
  EntityStore entityStore = new EntityStore();

  group("entity store", () {
    test("should be able to fetch an inserted entity by key", () {
      var entityKey = new Key()
        ..pathElement.add(new Key_PathElement()..kind="topLevel"..id=new Int64(43))
        ..pathElement.add(new Key_PathElement()..kind="2ndLevel"..id=new Int64(55));

      var prop1 = new Property()
          ..name = "prop1"
          ..value = (new Value()..integerValue = new Int64(0)..indexed=true);

      var entity = new Entity()
          ..key = entityKey
          ..property.add(prop1);
      entityStore.insert(entity);
      var lookupResult = entityStore.lookup(entityKey).entity;
      expect(lookupResult.key, entityKey);
      expect(lookupResult.property, [prop1]);
    });

    test("should be able to lookup a non-existent entity", () {
      var entityKey = new Key()
          ..pathElement.add(new Key_PathElement()..kind="topLevel"..id=new Int64(43))
          ..pathElement.add(new Key_PathElement()..kind="2ndLevel"..id=new Int64(56));

      var lookupResult = entityStore.lookup(entityKey).entity;
      expect(lookupResult.key, entityKey);
      expect(lookupResult.property, []);
    });

    test("should fail when trying to insert an entity with an incomplete path", () {
      var key1 = new Key()
          ..pathElement.add(new Key_PathElement()..id=new Int64(43));
      var ent1 = new Entity()..key = key1;
      expect(() => entityStore.insert(ent1), throws);

      var key2 = new Key()
          ..pathElement.add(new Key_PathElement()..kind="topLevel");
      var ent2 = new Entity()..key = key2;
      expect(() => entityStore.insert(ent2), throws);
    });
  });
}